@@ -11,7 +11,7 @@
 ##    Montasser Ghachem
 ##
 ## Last updated:
-##    2022-05-26
+##    2022-10-18
 ##
 ## License:
 ##    GPL 3
@@ -145,7 +145,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
                  tradinghours = 24, verbose = TRUE) {
 
   "
-@timebarsize  : the size of timebars in seconds default value: 1
+@timebarsize  : the size of timebars in seconds default value: 60
 @buckets      : number of buckets per volume of bucket size default value: 50
 @samplength   : sample length or window of buckets to calculate VPIN default
                 value: 50
@@ -166,22 +166,11 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   # -------------------------------------------------------------------------
   largs <- list(data, timebarsize, buckets, samplength, tradinghours, verbose)
   names(largs) <- names(formals())
-  largs$fn <- "vpin"
-  rst <- .xcheck$args(largs)
+  rst <- .xcheck$args(arglist = largs, fn = "vpin")
   ux$stopnow(rst$off, m = rst$error, s = uierrors$vpin()$fn)
 
-
-  if (missing(data))
-    ux$stopnow(m = vpin_err$missingdata, s = vpin_err$fn)
-
-  if (!is.data.frame(data) | ncol(data) < 3)
-    ux$stopnow(m = vpin_err$wrongdata, s = vpin_err$fn)
-
-  fargs <- c(timebarsize, buckets, samplength, tradinghours)
-
-  if (!is.numeric(fargs) | !all(ceiling(fargs) - floor(fargs) == 0))
-    ux$stopnow(m = vpin_err$wrongargs, s = vpin_err$fn)
-
+  # Convert data into a dataframe, in case it is a matrix or an array
+  data <- as.data.frame(data)
 
   # initialize the local variables
   tbv <- vbs <- bucket <- NULL
@@ -190,7 +179,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
 
   time_on <- Sys.time()
 
-  ux$show(m = vpin_ms$start)
+  ux$show(c= verbose, m = vpin_ms$start)
 
   ##############################################################################
   #             STEP 0 : CHECKING AND PREPARING THE DATA
@@ -200,7 +189,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   # USER MESSAGE
   # ----------------------------------------------------------------------------
 
-  ux$show(m = vpin_ms$step1)
+  ux$show(c= verbose, m = vpin_ms$step1)
 
   # We want only three columns: timestamp, price and volume so we import into
   # the dataset only the three first columns
@@ -222,7 +211,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
 
   rownames(dataset) <- NULL
 
-  # If the argument 'timebarsize' is larger than thte total duration of the
+  # If the argument 'timebarsize' is larger than the total duration of the
   # datasets in milliseconds, the abort.
   alltime <- 1000 * as.numeric(difftime(max(dataset$timestamp),
                                         min(dataset$timestamp), units = "secs"))
@@ -241,20 +230,21 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   # USER MESSAGE
   # ----------------------------------------------------------------------------
 
-  ux$show(m = vpin_ms$step2)
+  ux$show(c= verbose, m = vpin_ms$step2, skip = FALSE)
 
   # ----------------------------------------------------------------------------
   # I.1 CREATE THE VARIABLE INTERVAL
   # ----------------------------------------------------------------------------
 
   # create a variable called interval which contains the timebar extracted from
-  # the timestamp. If timebarsize ==1, then the observation of timestamp
+  # the timestamp. If timebarsize == 60, then the observation of timestamp
   # "2019-04-01 00:33:49" belong to the interval "2019-04-01 00:33:00", that is
-  # the timebar that starts at the minute 33 and lasts 1 minute (timebarsize)
+  # the timebar that starts at the minute 33 and lasts 1 minute (60 seconds)
+  # (timebarsize)
 
-  # If timebarsize ==5, then the observation of timestamp "2019-04-01 00:33:49"
+  # If timebarsize == 300, then the observation of timestamp "2019-04-01 00:33:49"
   # belong to the interval "2019-04-01 00:30:00", that is the timebar that
-  # starts at the minute 30 and lasts 5 minute (timebarsize)
+  # starts at the minute 30 and lasts 5 minutes (300 seconds) (timebarsize)
 
   tbsize <- paste(timebarsize, " sec", sep = "")
 
@@ -277,14 +267,17 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
     return(dsecs)
   }
 
-  if (nrow(dataset) >= 5000000) {
+  if (nrow(dataset) >= 50000) {
+
     temptime_on <- Sys.time()
 
-    tempbars <- aggregate(price ~ interval, data = dataset[1:10000, ],
+    chunk <- 5000
+
+    tempbars <- aggregate(price ~ interval, data = dataset[1:chunk, ],
       FUN = function(x) dp <- tail(x, 1) - head(x, 1)
     )
     tempbars <- merge(tempbars,
-                      aggregate(volume ~ interval, dataset[1:10000, ], sum),
+                      aggregate(volume ~ interval, dataset[1:chunk, ], sum),
                       by = "interval"
     )
     tempbars$interval <- as.POSIXct(tempbars$interval, tz = "")
@@ -292,10 +285,13 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
     temptime_off <- Sys.time()
 
     exptime <-  ux$timediff(temptime_on, temptime_off,
-                              (nrow(dataset) / (10000)))
+                              5*log2(nrow(dataset) / (chunk)))
 
-    cat("[~", ceiling(exptime), "seconds]\n")
+    ux$show(c= verbose, m = paste("[~", ceiling(exptime), "seconds]"))
 
+  } else {
+
+    ux$show(c= verbose, m = "")
   }
 
   # -                                                                          -
@@ -320,7 +316,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   # USER MESSAGE
   # --------------------------------------------------------------------------
 
-  ux$show(m = vpin_ms$step3)
+  ux$show(c= verbose, m = vpin_ms$step3)
 
   # --------------------------------------------------------------------------
   # II.1 CALCULATE NUMBER OF DAYS (NDAYS), TOTAL VOLUME (TOTVOL) AND THEN VBS
@@ -338,7 +334,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
 
   params <- c(timebarsize, buckets, samplength, vbs, ndays)
 
-  names(params) <- c("tbSize", "buckets", "samplength", "VBS", "#days")
+  names(params) <- c("tbSize", "buckets", "samplength", "VBS", "ndays")
 
   estimatevpin@parameters <- params
 
@@ -356,7 +352,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   # USER MESSAGE
   # --------------------------------------------------------------------------
 
-  ux$show(m = vpin_ms$step4)
+  ux$show(c= verbose, m = vpin_ms$step4)
 
   # --------------------------------------------------------------------------
   # TASK DESCRIPTON
@@ -418,6 +414,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
 
   largebars <- subset(minutebars, tbv > threshold)
 
+  largebnum <- NULL
   if (nrow(largebars) != 0)
     largebnum <- which(minutebars$id %in% largebars$id)
 
@@ -454,51 +451,55 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   # -------------------------------
   #   2019-04-02 04:53  53.0    340.
 
-  minutebars[largebnum, ]$tbv <- minutebars[largebnum, ]$tbv %% threshold
+  if(!is.null(largebnum)) {
 
-  # We will now replicate the time bar with the same price movement (dp), the
-  # same interval but with trade volume equal to threshold/x (x=10 in our case).
-  # The number of replications we need is the integer division of (tbv) by
-  # (threshold/x). n_rep stores these numbers of replication.
-  # This number for each of these rows is the integer division of (tbv) by
-  # (threshold/x), i.e., largebars$tbv %/% threshold
+    minutebars[largebnum, ]$tbv <- minutebars[largebnum, ]$tbv %% threshold
 
-  n_rep <- x * largebars$tbv %/% threshold
+    # We will now replicate the time bar with the same price movement (dp), the
+    # same interval but with trade volume equal to threshold/x (x=10 in our case).
+    # The number of replications we need is the integer division of (tbv) by
+    # (threshold/x). n_rep stores these numbers of replication.
+    # This number for each of these rows is the integer division of (tbv) by
+    # (threshold/x), i.e., largebars$tbv %/% threshold
 
-  # All what is left now is to change the value of 'tbv' in 'largebars' to
-  # (threshold/x) and then replicate the timebars using the corresponding value
-  # in n_rep
+    n_rep <- x * largebars$tbv %/% threshold
 
-  largebars$tbv <- threshold / x
+    # All what is left now is to change the value of 'tbv' in 'largebars' to
+    # (threshold/x) and then replicate the timebars using the corresponding value
+    # in n_rep
 
-  largebars <- largebars[rep(seq_len(nrow(largebars)), n_rep), ]
+    largebars$tbv <- threshold / x
 
-  rm(n_rep)
+    largebars <- largebars[rep(seq_len(nrow(largebars)), n_rep), ]
 
-  # In our example, largebars will contain 50 replicated timebars of the
-  # following timebar:
-  #
-  #   minute              dp    tbv
-  # -------------------------------
-  #   2019-04-02 04:53  53.0    100.
-  #
-  # Since threshold is 1000 so threshold/x = 1000/10=100; so each of our
-  # timebars should have a volume of 1000. Since we have already placed the
-  # remainder (340) in the original dataset, we just need to distribute the
-  # remaining volume 5000 into timebars with a volume of 100, which
-  # gives us 50 such timebars
+    rm(n_rep)
 
-  # The last step now is to add these rows to the main dataset and sort it by
-  # interval so that all timebars of the same interval will be neighbors.
+    # In our example, largebars will contain 50 replicated timebars of the
+    # following timebar:
+    #
+    #   minute              dp    tbv
+    # -------------------------------
+    #   2019-04-02 04:53  53.0    100.
+    #
+    # Since threshold is 1000 so threshold/x = 1000/10=100; so each of our
+    # timebars should have a volume of 1000. Since we have already placed the
+    # remainder (340) in the original dataset, we just need to distribute the
+    # remaining volume 5000 into timebars with a volume of 100, which
+    # gives us 50 such timebars
 
-  minutebars <- rbind(minutebars, largebars)
-  minutebars <- minutebars[order(minutebars$interval), ]
+    # The last step now is to add these rows to the main dataset and sort it by
+    # interval so that all timebars of the same interval will be neighbors.
 
-  # Finally, we reassign new identifiers to timebars (id) to make it easier to
-  # identify them in coming tasks
+    minutebars <- rbind(minutebars, largebars)
+    minutebars <- minutebars[order(minutebars$interval), ]
 
-  minutebars$id <- seq.int(nrow(minutebars))
-  rm(largebars, largebnum)
+    # Finally, we reassign new identifiers to timebars (id) to make it easier to
+    # identify them in coming tasks
+
+    minutebars$id <- seq.int(nrow(minutebars))
+    rm(largebars, largebnum)
+
+  }
 
   ############################################################################
   #             STEP 4 : ASSIGNING T-MINUTE TIME BARS INTO BUCKETS
@@ -508,7 +509,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   # USER MESSAGE
   # --------------------------------------------------------------------------
 
-  ux$show(m = vpin_ms$step5)
+  ux$show(c= verbose, m = vpin_ms$step5)
 
   # --------------------------------------------------------------------------
   # IV.1 ASSIGN A BUCKET TO EACH TIMEBAR | FIND EXCESS VOLUME FOR EACH OF THEM
@@ -546,7 +547,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   # USER MESSAGE
   # --------------------------------------------------------------------------
 
-  ux$show(m = vpin_ms$step6)
+  ux$show(c= verbose, m = vpin_ms$step6)
 
   # --------------------------------------------------------------------------
   # TASK DESCRIPTON
@@ -658,7 +659,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   # USER MESSAGE
   # --------------------------------------------------------------------------
 
-  ux$show(m = vpin_ms$step7)
+  ux$show(c= verbose, m = vpin_ms$step7)
 
   # --------------------------------------------------------------------------
   # VI.1 ASSIGN N(0, 1) PROB. TO EACH TIMEBAR AND CALCULATE BUY/SELL VOLUMES
@@ -690,7 +691,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   # ++++++++++++++++++++
   # agg.bvol    : sum of buy volume (bvol) per bucket
   # agg.svol    : sum of sell volume (svol) per bucket
-  # OI     : the difference between agg.bvol and agg.svol
+  # OI          : the difference between agg.bvol and agg.svol
   # init.time   : the first timebar in the bucket
   # final.time  : the last timebar in the bucket
   # +++++++++++++++++++
@@ -713,7 +714,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   # USER MESSAGE
   # --------------------------------------------------------------------------
 
-  ux$show(m = vpin_ms$step8)
+  ux$show(c= verbose, m = vpin_ms$step8)
 
   # --------------------------------------------------------------------------
   # TASK DESCRIPTON
@@ -746,7 +747,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
 
     estimatevpin@errorMessage <- vpin_err$largesamplength
 
-    ux$show(m = vpin_ms$aborted)
+    ux$show(c= verbose, m = vpin_ms$aborted)
 
     ux$show(ux$line())
 
@@ -761,8 +762,8 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
     (samplength * vbs)
   bucketdata$vpin[samplength] <- bucketdata$cumoi[samplength] /
     (samplength * vbs)
-  bucketdata$duration <- as.numeric(difftime(bucketdata$endtime,
-                                             bucketdata$starttime, units = "secs"))
+  bucketdata$duration <- as.numeric(
+    difftime(bucketdata$endtime, bucketdata$starttime, units = "secs"))
 
   # --------------------------------------------------------------------------
   # VII.2 CORRECTING THE DURATION VECTOR
@@ -799,6 +800,8 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
   dailyvpin <- setNames(aggregate(vpin ~ day, bucketdata,
                         function(x) mean(x, na.rm = TRUE)), c("day", "dvpin"))
 
+  dailyvpin$day <- as.Date(dailyvpin$day)
+
   temp <- setNames(aggregate(cbind(duration, vpindur) ~ day, bucketdata,
                             function(x) sum(x, na.rm = TRUE)),
                                       c("day", "sumD", "sumvD"))
@@ -821,7 +824,7 @@ vpin <- function(data, timebarsize = 60, buckets = 50, samplength = 50,
 
   estimatevpin@runningtime <- ux$timediff(time_on, time_off)
 
-  ux$show(m = vpin_ms$complete)
+  ux$show(c= verbose, m = vpin_ms$complete)
 
   return(estimatevpin)
 }
